pro chg_domain, fbb, fspect, tbb, fft_fspect, f_to_t, bw=bw, dft=dft
;+
;Convert from Freq Domain to Time Domain if F_TO_T=1, or time to freq if
;       F_TO_T=0. use FFT unless DFT is set.
;
;CHG_DOMAIN, fbb, fspect, tbb, fft_fspect, f_to_t, bw=bw, dft=dft
;
;NOTE 1: This can go from freq domain to time domain (F_TO_T=1), or
;vice-versa (F_TO_T=0). For F_TO_T=1, FBB and FSPECT are inputs and TBB,
;FFT_FSPECT are outputs; and vice-versa. No matter which direction
;(i.e., whether F_TO_T is 1 or 0), the two frequency-domain arrays
;always are the first two parameters and the two time-domain arrays the
;second two parameters.
;
;NOTE 2. The lengths (N64 below) of both FBB and TBB must be even,
;usually (but not necessarily) a power of two, uniformly spaced, and
;have 0 in the middle. For example, If the spacing of FBB is 2 and
;N64=8, then FBB=[-8, -6, -4, -2, 0, 2, 4, 6].
;
;INPUTS (assuming F_TO_T=1):
;FBB, the array of baseband freqs in the freq domain [4096]. If BW is
;   specified, or if f_to_t=0 and tbb is specified, enter a dummy for FBB
;   because it calculates FBB and TBB using BW.
;FSPECT, the signal array in the frequency domain [4096] TBB, the array
;   of times in the time domain [4096]. If BW is specified, or if f_to_t=1
;   and fbb is speciffied, enter a dummy for TBB because it calculates FBB
;   and TBB using BW.
;F_TO_T, if 1 we go from freq to time, if 0 from time to freq.
;
;OUTPUTS  (assuming F_TO_T=1):
;TBB, the array of times for the time series signal array.
;FFT_FSPECT, the time series signal array [complex 4096].
;
;KEYWORDS:
;BW, the bandwidth of the frequency spectrum. Set this ONLY if
;F_TO_T=1 and you don't want to calculate FBB yourself. If set,
;it will return FBB (and TBB, FFT_FPECT) as output arrays.
;
;DFT, if set, use DFT instead of FFT to do the FT. Much slower!  Don't
;   set DFT--it was used during development of this code. .
;
;HISTORY: 
;07FEB2019, Carl fixed a bug calculating TBB and updated documentation.
;-

;if f_to_t eq 1 then n64=n_elements(fspect) else n64=n_elements(fft_fspect)
if f_to_t eq 1 then begin
   n64=n_elements(fspect)
   if (n_elements( bw) eq 1) then begin
      delfrq= double(bw)/ n64
      fbb= delfrq * (dindgen( n64)- n64/2l)
   endif else begin
      delfrq= double(max(fbb)-min(fbb))/(n64-1)
      bw=n64*delfrq
   endelse
   tsmpl= 1./double(bw)
   delt= tsmpl
   tbb=  delt * (dindgen( n64)- n64/2l)
endif else begin                ;if f_to_t=0 do the following:
   n64=n_elements(fft_fspect)
   tsmpl= double(max(tbb)-min(tbb))/(n64-1)
   fbb= (dindgen( n64)- n64/2l)/(tsmpl*n64)
endelse

;=====================
goto, skip
if f_to_t eq 1 then n64=n_elements(fspect) else n64=n_elements(fft_fspect)
if ( (n_elements( fbb) ne n64) or $
   (n_elements( tbb) ne n64) or $
   (n_elements( bw) eq 1) )then begin
;GENERATE BB FREQS AND TIMES...these have 0 in the middle in both
;domains.           
delfrq= bw/ n64
fbb= delfrq * (dindgen( n64)- n64/2l)

tsmpl= 1./bw
delt= tsmpl
tbb=  delt * (dindgen( n64)- n64/2l)
endif
skip:
;==========================

if keyword_set( dft) then begin
;USE DFT
   if f_to_t eq 1 then begin
      dft, fbb, fspect, tbb, fft_fspect
   endif else begin
      dft, fbb, fspect, tbb, fft_fspect, /inverse
   endelse
   
endif else begin
;USE FFT
   if f_to_t eq 1 then begin
      fft_fspect= shift( fft( shift( fspect, n64/2)), n64/2)
   endif else begin
      fspect=     shift( fft( shift( fft_fspect, n64/2), /inverse), n64/2)
endelse

   endelse
;stop

return
end


